/**
 * Created by Sam R.
 * on 30-04-2011
 * at 10:49
 */
package com.cirrial.ld20 {
import com.cirrial.ld20.characters.*;
import com.cirrial.ld20.widgets.*;


import org.flixel.*;

public class PlayState extends FlxState {

    public static var level:Level_FinalLevel;
    public static var hud:Hud;
    public static var player:Player;

    public static var won:Boolean = false;

    private var currentSpawn:SpawnPoint;


    override public function create():void {
        level = new Level_FinalLevel(true, onObjectAddedCallback);
        attachPaths();
        addEnemySubgroups();
        player.setRespawn(currentSpawn);
        level.layerMainMap.follow();
        FlxG.camera.follow(player, FlxCamera.STYLE_PLATFORMER);
        FlxG.camera.setBounds(-Main.HUD_WIDTH, -Main.HUD_WIDTH, level.layerMainMap.width + Main.HUD_WIDTH, level.layerMainMap.height + Main.HUD_CHATTER_HEIGHT);
        var partner:Partner = new Partner();
        player.setPartner(partner);
        level.PlayerGroup.add(partner);

        hud = new Hud();
        add(hud);
    }

    override public function update():void {
        FlxG.collide(level.PlayerGroup, level.layerMainMap);
        FlxG.collide(level.EnemiesGroup, level.layerMainMap);
        if(player.partner.deployed) FlxG.collide(level.PlayerGroup, level.PlayerGroup);
        FlxG.collide(level.PlayerGroup, level.StageWidgetsGroup, onCharacterWidgetCollision);
        FlxG.collide(level.EnemiesGroup, level.StageWidgetsGroup, onCharacterWidgetCollision);
        FlxG.collide(level.PlayerGroup, level.EnemiesGroup, onPlayerEnemyCollision);
        FlxG.overlap(level.PlayerGroup, level.BackgroundWidgetsGroup, onPlayerWidgetOverlap);
        FlxG.overlap(level.PlayerGroup, level.ForegroundWidgetsGroup, onPlayerWidgetOverlap);
        if(!player.alive && player.respawnTimer > 0) {
            player.updateRespawnTimer();
        } else {
            hud.healthFade.alpha = 0.8 - 0.8*(player.health/100);
        }
        super.update();
    }

    public function onObjectAddedCallback(obj:Object, group:FlxGroup, lvl:BaseLevel, scrollX:Number, scrollY:Number, properties:Array):Object {
        var s:String;
        if(obj is Player) {
            player = obj as Player;
        } else if (obj is EnemySpawner) {
            var enemySpawn:EnemySpawner = obj as EnemySpawner;
            var enemyType:String = "";
            var enemyNum:Number = 0;
            var enemyTime:Number = 0;
            for(s in properties) {
                switch(properties[s].name) {
                    case "EnemyType":
                        enemyType = properties[s].value;
                        break;
                    case "EnemyNum":
                        enemyNum = properties[s].value;
                        break;
                    case "EnemyTime":
                        enemyTime = properties[s].value;
                        break;
                    case "InitiallyOff":
                        if(properties[s].value) enemySpawn.deactivate();
                        break;
                }
            }
            enemySpawn.createEnemyGroup(enemyType, enemyNum, enemyTime);

        } else if (obj is SpawnPoint) {
            var spawn:SpawnPoint = obj as SpawnPoint;
//            var makeFirstSpawn:Boolean = false;
//            for(s in properties) {
//                if(properties[s].name == "FirstSpawn") {
//                     makeFirstSpawn = properties[s].value;
//                }
//            }
            if(properties["0"].value) {//makeFirstSpawn) {
                currentSpawn = spawn;
                currentSpawn.activate();
            }
        } else if (obj is ChatterTrigger) {
            var chat:ChatterTrigger = obj as ChatterTrigger;
            var name:String = "";
            var message:String = "";
            for(s in properties) {
                switch(properties[s].name) {
                    case "Name":
                        name = properties[s].value;
                        break;
                    case "Message":
                        message = properties[s].value;
                        break;
                }
            }
            chat.setProperties(name, message);
        } else if(obj is ObjectLink) {
            var link:ObjectLink = obj as ObjectLink;
            var fromObject:Object = link.fromObject;
            if(fromObject is Switch) {
                var fromSwitch:Switch = link.fromObject as Switch;
                if(link.toObject is IActiveatable) fromSwitch.addWidget(link.toObject as IActiveatable);
            }
        }
        return obj;
    }

    public function attachPaths():void {
        for each (var pathData:PathData in level.paths) {
            if(pathData.childSprite) {
                if(pathData.childSprite is Elevator) {
                    var elevator:Elevator = pathData.childSprite as Elevator;
                    elevator.setPathNodes(pathData.nodes);
                } else if(pathData.childSprite is TriggeredPlatform) {
                    var trigPlat:TriggeredPlatform = pathData.childSprite as TriggeredPlatform;
                    trigPlat.setPathNodes(pathData.nodes);
                }
            }
        }
    }

    public function addEnemySubgroups():void {
        for each(var sprite:FlxSprite in level.StageWidgetsGroup.members) {
            if(sprite is EnemySpawner) {
                var enemySpawner:EnemySpawner = sprite as EnemySpawner;
                enemySpawner.addEnemySubgroupToLevel(level.EnemiesGroup);
            }

        }
    }


    public function onCharacterWidgetCollision(character:FlxObject, widget:FlxObject):void {
        if(widget is EnergyPool) {
            character.hurt(200 * FlxG.elapsed);
        } else if (widget is OneWaySurface) {
            var surface:OneWaySurface = widget as OneWaySurface;
            if(((character.velocity.x > 0) && (surface.facing == FlxObject.LEFT)) ||
                    ((character.velocity.x < 0) && (surface.facing == FlxObject.RIGHT))) {
                character.hurt(200 * FlxG.elapsed);
            }
            if(character.velocity.x == 0) { // no, players, you are not allowed to stand still and regenerate
                character.hurt(20 * FlxG.elapsed);
            }
        } else if(widget is LaunchPad) {
            var launchPad:LaunchPad = widget as LaunchPad;
            launchPad.fling(character);
        }
    }

    public function onPlayerEnemyCollision(p:FlxObject, e:FlxObject):void {
        if(p is Player) {
            if(!p.flickering) {
                p.hurt(20);
                p.flicker(1.0);
                if(p.x < e.x) {
                   p.velocity.x -= 200;
                } else {
                   p.velocity.x += 200;
                }
                p.velocity.y -= 100;
            }
        } else if(p is Partner) {
            e.hurt(5);
            if(p.x < e.x) {
               e.velocity.x += 200;
            } else {
               e.velocity.x -= 200;
            }
            e.velocity.y -= 100;
        }
    }

    public function onPlayerWidgetOverlap(object:FlxObject, widget:FlxObject):void {
        if(object is Player) {
            var _player:Player = object as Player;
            if(widget is Switch) {
                if(FlxG.keys.justPressed("Z")) {
                    var _switch:Switch = widget as Switch;
                    _switch.toggle();
                }
            } else if(widget is Exit) {
                _player.end();
                FlxG.fade(0xFF000000, 3, onFade);
            } else if(widget is SpawnPoint) {
                var spawn:SpawnPoint = widget as SpawnPoint;
                if(spawn != currentSpawn) {
                    currentSpawn.deactivate();
                    currentSpawn = spawn;
                    currentSpawn.activate();
                    player.setRespawn(currentSpawn);
                }
            } else if(widget is ChatterTrigger) {
                var chat:ChatterTrigger = widget as ChatterTrigger;
                hud.showMessage(chat.name, chat.message);
                chat.kill();
            }
        }
    }

    public function onFade():void {
        won = true;
        FlxG.switchState(new CutsceneState());
    }
}
}
